@@ -1,20 +1,8 @@
-/*****************************************************************************
- * Copyright (c) 2011-2012 Sven Brauch <svenbrauch@googlemail.com>           *
- *                                                                           *
- * This program is free software; you can redistribute it and/or             *
- * modify it under the terms of the GNU General Public License as            *
- * published by the Free Software Foundation; either version 2 of            *
- * the License, or (at your option) any later version.                       *
- *                                                                           *
- * This program is distributed in the hope that it will be useful,           *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
- * GNU General Public License for more details.                              *
- *                                                                           *
- * You should have received a copy of the GNU General Public License         *
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
- *****************************************************************************
- */
+/*
+    SPDX-FileCopyrightText: 2011-2012 Sven Brauch <svenbrauch@googlemail.com>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "context.h"
 
@@ -175,7 +163,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::functionCallI
     if ( depth() != 1 || ! functionCalled ) {
         return resultingItems;
     }
-    if ( DUContext* args = DUChainUtils::getArgumentContext(functionCalled) ) {
+    if ( DUContext* args = DUChainUtils::argumentContext(functionCalled) ) {
         int normalParameters = args->localDeclarations().count() - functionCalled->defaultParametersSize();
         if ( normalParameters > m_alreadyGivenParametersCount ) {
             qCDebug(KDEV_PYTHON_CODECOMPLETION) << "Not at default arguments yet";
@@ -218,7 +206,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::defineItems()
 
     bool isOwnContext = true;
     foreach ( DUContext* c, baseClassContexts ) {
-        QList<DeclarationDepthPair> declarations = c->allDeclarations(
+        const auto declarations = c->allDeclarations(
             CursorInRevision::invalid(), m_duContext->topContext(), false
         );
         foreach ( const DeclarationDepthPair& d, declarations ) {
@@ -234,7 +222,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::defineItems()
                 }
                 existingIdentifiers << identifier;
                 QStringList argumentNames;
-                DUContext* argumentsContext = DUChainUtils::getArgumentContext(funcDecl);
+                DUContext* argumentsContext = DUChainUtils::argumentContext(funcDecl);
                 if ( argumentsContext ) {
                     foreach ( Declaration* argument, argumentsContext->localDeclarations() ) {
                         argumentNames << argument->identifier().toString();
@@ -266,7 +254,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::raiseItems()
     }
     Declaration* base = declarations.first();
     IndexedType baseType = base->abstractType()->indexed();
-    QList<DeclarationDepthPair> validDeclarations;
+    QVector<DeclarationDepthPair> validDeclarations;
     ClassDeclaration* current = nullptr;
     StructureType::Ptr type;
     auto decls = m_duContext->topContext()->allDeclarations(CursorInRevision::invalid(), m_duContext->topContext());
@@ -305,7 +293,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::inheritanceIt
     ItemList resultingItems;
     DUChainReadLocker lock;
     qCDebug(KDEV_PYTHON_CODECOMPLETION) << "InheritanceCompletion";
-    QList<DeclarationDepthPair> declarations;
+    QVector<DeclarationDepthPair> declarations;
     if ( ! m_guessTypeOfExpression.isEmpty() ) {
         // The class completion is a member access
         auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
@@ -321,7 +309,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::inheritanceIt
     else {
         declarations = m_duContext->allDeclarations(m_position, m_duContext->topContext());
     }
-    QList<DeclarationDepthPair> remainingDeclarations;
+    QVector<DeclarationDepthPair> remainingDeclarations;
     foreach ( const DeclarationDepthPair& d, declarations ) {
         Declaration* r = Helper::resolveAliasDeclaration(d.first);
         if ( r && r->topContext() == Helper::getDocumentationFileContext() ) {
@@ -485,7 +473,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::classMemberIn
     if ( ! decl ) {
         return resultingItems;
     }
-    DUContext* args = DUChainUtils::getArgumentContext(duContext()->owner());
+    DUContext* args = DUChainUtils::argumentContext(duContext()->owner());
     if ( ! args ) {
         return resultingItems;
     }
@@ -534,7 +522,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::generatorItem
     if ( v->unknownNames().size() >= 2 ) {
         // we only take the first two, and only two. It gets too much items otherwise.
         QStringList combinations;
-        auto names = v->unknownNames().toList();
+        auto names = v->unknownNames().values();
         combinations << names.at(0) + ", " + names.at(1);
         combinations << names.at(1) + ", " + names.at(0);
         foreach ( const QString& c, combinations ) {
@@ -612,7 +600,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
             return ItemList();
         }
         DUChainReadLocker lock;
-        QList<DeclarationDepthPair> declarations = m_duContext->allDeclarations(m_position, m_duContext->topContext());
+        auto declarations = m_duContext->allDeclarations(m_position, m_duContext->topContext());
         foreach ( const DeclarationDepthPair& d, declarations ) {
             if ( d.first && d.first->context()->type() == DUContext::Class ) {
                 declarations.removeAll(d);
@@ -676,7 +664,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::getMissingIncludeI
     return items;
 }
 
-QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToItemList(QList<DeclarationDepthPair> declarations, int maxDepth)
+QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToItemList(const QVector<DeclarationDepthPair>& declarations, int maxDepth)
 {
     QList<CompletionTreeItemPointer> items;
     
@@ -712,7 +700,8 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToI
 
 QList< CompletionTreeItemPointer > PythonCodeCompletionContext::declarationListToItemList(QList< Declaration* > declarations)
 {
-    QList<DeclarationDepthPair> fakeItems;
+    QVector<DeclarationDepthPair> fakeItems;
+    fakeItems.reserve(declarations.size());
     foreach ( Declaration* d, declarations ) {
         fakeItems << DeclarationDepthPair(d, 0);
     }
@@ -780,10 +769,10 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::getCompletionItems
     }
     // the PublicOnly will filter out non-explictly defined __get__ etc. functions inherited from object
     auto searchContexts = Helper::internalContextsForClass(cls, m_duContext->topContext(), Helper::PublicOnly);
-    QList<DeclarationDepthPair> keepDeclarations;
+    QVector<DeclarationDepthPair> keepDeclarations;
     foreach ( const DUContext* currentlySearchedContext, searchContexts ) {
         qCDebug(KDEV_PYTHON_CODECOMPLETION) << "searching context " << currentlySearchedContext->scopeIdentifier() << "for autocompletion items";
-        QList<DeclarationDepthPair> declarations = currentlySearchedContext->allDeclarations(CursorInRevision::invalid(),
+        const auto declarations = currentlySearchedContext->allDeclarations(CursorInRevision::invalid(),
                                                                                                 m_duContext->topContext(),
                                                                                                 false);
         qCDebug(KDEV_PYTHON_CODECOMPLETION) << "found" << declarations.length() << "declarations";
