@@ -1,23 +1,11 @@
-/*****************************************************************************
- * Copyright (c) 2007 Piyush verma <piyush.verma@gmail.com>                  *
- * Copyright 2007 Andreas Pakulat <apaku@gmx.de>                             *
- * Copyright 2010-2016 Sven Brauch <svenbrauch@googlemail.com>               *
- * Copyright 2016 Francis Herne <mail@flherne.uk>                            *
- *                                                                           *
- * This program is free software; you can redistribute it and/or             *
- * modify it under the terms of the GNU General Public License as            *
- * published by the Free Software Foundation; either version 2 of            *
- * the License, or (at your option) any later version.                       *
- *                                                                           *
- * This program is distributed in the hope that it will be useful,           *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
- * GNU General Public License for more details.                              *
- *                                                                           *
- * You should have received a copy of the GNU General Public License         *
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
- *****************************************************************************
- */
+/*
+    SPDX-FileCopyrightText: 2007 Piyush verma <piyush.verma@gmail.com>
+    SPDX-FileCopyrightText: 2007 Andreas Pakulat <apaku@gmx.de>
+    SPDX-FileCopyrightText: 2010-2016 Sven Brauch <svenbrauch@googlemail.com>
+    SPDX-FileCopyrightText: 2016 Francis Herne <mail@flherne.uk>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "declarationbuilder.h"
 #include "duchain/declarations/functiondeclaration.h"
@@ -1139,9 +1127,9 @@ void DeclarationBuilder::assignToSubscript(SubscriptAst* subscript, const Declar
     }
     auto map = MapType::Ptr::dynamicCast(list);
     if ( map ) {
-        if ( subscript->slice && subscript->slice->astType == Ast::IndexAstType ) {
+        if ( subscript->slice && subscript->slice->astType != Ast::SliceAstType) {
             ExpressionVisitor keyVisitor(currentContext());
-            keyVisitor.visitNode(static_cast<IndexAst*>(subscript->slice)->value);
+            keyVisitor.visitNode(subscript->slice);
             AbstractType::Ptr key = keyVisitor.lastType();
             if ( key ) {
                 map->addKeyType<Python::UnsureType>(key);
@@ -1354,6 +1342,14 @@ void DeclarationBuilder::visitAnnotationAssignment(AnnotationAssignmentAst* node
     assignToUnknown(node->target, assignType);
 }
 
+void DeclarationBuilder::visitAssignmentExpression(AssignmentExpressionAst* node) {
+    AstDefaultVisitor::visitAssignmentExpression(node);
+
+    ExpressionVisitor v(currentContext());
+    v.visitNode(node->value);
+    assignToUnknown(node->target, v.lastType());
+}
+
 void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
 {
     visitNodeList(node->decorators);
@@ -1744,8 +1740,14 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
     int defaultParametersCount = node->defaultValues.length();
     int parametersCount = node->arguments.length();
     int firstDefaultParameterOffset = parametersCount - defaultParametersCount;
+
+    int defaultKwParametersCount = node->defaultKwValues.length();
+    int kwonlyCount = node->kwonlyargs.length();
+    int posonlyCount = node->posonlyargs.length();
+    int totalArgCount = parametersCount + posonlyCount + kwonlyCount;
+    int firstDefaultKwParameterOffset = totalArgCount - defaultKwParametersCount;
     int currentIndex = 0;
-    foreach ( ArgAst* arg, node->arguments + node->kwonlyargs ) {
+    foreach ( ArgAst* arg, node->posonlyargs + node->arguments + node->kwonlyargs ) {
         // Iterate over all the function's arguments, create declarations, and add the arguments
         // to the functions FunctionType.
         currentIndex += 1;
@@ -1796,6 +1798,18 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
             // TODO add the real expression from the document here as default value
             workingOnDeclaration->addDefaultParameter(IndexedString("..."));
         }
+        else if ( currentIndex > firstDefaultKwParameterOffset && currentIndex <= totalArgCount ) {
+            // Handle kw only arguments with default values, like def foo(*, bar = 3): pass
+            // Find type of given default value, and assign it to the declaration
+            ExpressionVisitor v(currentContext());
+            v.visitNode(node->defaultKwValues.at(currentIndex - firstDefaultKwParameterOffset - 1));
+            if ( v.lastType() ) {
+                argumentType = v.lastType();
+            }
+            // TODO add the real expression from the document here as default value
+            workingOnDeclaration->addDefaultParameter(IndexedString("..."));
+        }
+
 
         if ( isFirst && ! workingOnDeclaration->isStatic() && currentContext() && currentContext()->parentContext() ) {
             DUChainReadLocker lock;
