@@ -1,26 +1,9 @@
-/*****************************************************************************
- * Copyright 2010 (c) Miquel Canes Gonzalez <miquelcanes@gmail.com>          *
- * Copyright 2012 (c) Sven Brauch <svenbrauch@googlemail.com>                *
- *                                                                           *
- * Permission is hereby granted, free of charge, to any person obtaining     *
- * a copy of this software and associated documentation files (the           *
- * "Software"), to deal in the Software without restriction, including       *
- * without limitation the rights to use, copy, modify, merge, publish,       *
- * distribute, sublicense, and/or sell copies of the Software, and to        *
- * permit persons to whom the Software is furnished to do so, subject to     *
- * the following conditions:                                                 *
- *                                                                           *
- * The above copyright notice and this permission notice shall be            *
- * included in all copies or substantial portions of the Software.           *
- *                                                                           *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,           *
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF        *
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                     *
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE    *
- * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION    *
- * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION     *
- * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.           *
- *****************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2010 Miquel Canes Gonzalez <miquelcanes@gmail.com>
+    SPDX-FileCopyrightText: 2012 Sven Brauch <svenbrauch@googlemail.com>
+
+    SPDX-License-Identifier: MIT
+*/
 
 #include <QDebug>
 #include "duchaindebug.h"
@@ -162,7 +145,7 @@ ReferencedTopDUContext PyDUChainTest::parse(const QString& code)
     TestFile* testfile = new TestFile(code + "\n", "py", nullptr, testDir.absolutePath().append("/"));
     createdFiles << testfile;
 
-    testfile->parse((TopDUContext::Features) (TopDUContext::ForceUpdate | TopDUContext::AST) );
+    testfile->parse(TopDUContext::ForceUpdate | TopDUContext::AST);
     testfile->waitForParsed(2000);
     
     if ( testfile->isReady() ) {
@@ -450,6 +433,8 @@ void PyDUChainTest::testCrashes_data() {
 #endif
     QTest::newRow("definition_in_baseclass_1") << "class Foo(lambda x: 1): pass";
     QTest::newRow("definition_in_baseclass_2") << "class Foo([x for x in (1, 2)]): pass";
+    QTest::newRow("match") << "match x.split():\n case [a, b]: pass";
+    QTest::newRow("except_star") << "try: pass\nexcept* (IndexError, ValueError): pass";
 }
 
 void PyDUChainTest::testClassVariables()
@@ -840,6 +825,8 @@ void PyDUChainTest::testTypes()
     QEXPECT_FAIL("property_wrong", "visitCall uses declaration if no type", Continue);
     QEXPECT_FAIL("property_setter", "very basic property support", Continue);
     QEXPECT_FAIL("assignment_expr_context", "not implemented", Continue);
+    if (!visitor->found)
+        qDebug() << m_ast->dump();
     QCOMPARE(visitor->found, true);
 }
 
@@ -898,6 +885,14 @@ void PyDUChainTest::testTypes_data()
 
     QTest::newRow("arg_after_vararg") << "def func(x, y, *, z:int): return z\ncheckme = func()" << "int";
     QTest::newRow("arg_after_vararg_with_default") << "def func(x=5, y=3, *, z:int): return z\ncheckme = func()" << "int";
+    QTest::newRow("arg_default") << "def func(x=5, z='foo'): return z\ncheckme = func()" << "str";
+    QTest::newRow("arg_kw_no_default") << "def func(x=5, *, a, z='foo'): return a\ncheckme = func()" << "mixed";
+    QTest::newRow("arg_kw_default_after_no_default") << "def func(x=5, *, a, z='foo'): return z\ncheckme = func()" << "str";
+    QTest::newRow("arg_kw_default") << "def func(x=5, *, z='foo'): return z\ncheckme = func()" << "str";
+    QTest::newRow("arg_kw_default_after_default") << "def func(x=5, *, y='foo', z=True): return z\ncheckme = func()" << "bool";
+    QTest::newRow("kw_default_before_default") << "def func(*, y='foo', z=True): return y\ncheckme = func()" << "str";
+    QTest::newRow("kw_default_after_no_default") << "def func(*, y, z=True): return y\ncheckme = func()" << "mixed";
+    QTest::newRow("kw_default_after_default") << "def func(*, y='foo', z=True): return z\ncheckme = func()" << "bool";
 
     QTest::newRow("class_scope_end_inside") << "a = str()\nclass M:\n"
                                                "  a = 2\n  foo = a\n"
