@@ -1,23 +1,9 @@
-/***************************************************************************
- *   This file is part of KDevelop                                         *
- *   Copyright 2007 Andreas Pakulat <apaku@gmx.de>                         *
- *   Copyright 2012 Patrick Spendrin <ps_ml@gmx.de>                        *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
- *   published by the Free Software Foundation; either version 2 of the    *
- *   License, or (at your option) any later version.                       *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
- *   License along with this program; if not, write to the                 *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2007 Andreas Pakulat <apaku@gmx.de>
+    SPDX-FileCopyrightText: 2012 Patrick Spendrin <ps_ml@gmx.de>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 // The Python 3.4 Language Reference was used as basis for this AST
 
@@ -86,6 +72,7 @@ public:
         WithItemAstType,
         RaiseAstType,
         TryAstType,
+        TryStarAstType,
         ImportAstType,
         ImportFromAstType,
         GlobalAstType,
@@ -94,9 +81,12 @@ public:
         AssertionAstType,
         AugmentedAssignmentAstType,
         AnnotationAssignmentAstType,
+        MatchAstType,
         LastStatementType,
+
         ExpressionAstType, // everything below is an expression
         AwaitAstType,
+        ConstantAstType,
         NameAstType,
         NameConstantAstType,
         CallAstType,
@@ -130,6 +120,19 @@ public:
         AssignmentExpressionAstType,
         LastExpressionType, // keep this at the end of the expr ast list
 
+        // everything below is a pattern
+        MatchCaseAstType,
+        PatternAstType,
+        MatchValueAstType,
+        MatchSingletonAstType,
+        MatchSequenceAstType,
+        MatchMappingAstType,
+        MatchClassAstType,
+        MatchStarAstType,
+        MatchAsAstType,
+        MatchOrAstType,
+        LastPatternType,
+
         CodeAstType,
         ExceptionHandlerAstType,
         AliasAstType, // for imports
@@ -186,6 +189,7 @@ public:
 
     Ast(Ast* parent, AstType type);
     Ast();
+    virtual ~Ast() {};
     Ast* parent = nullptr;
     AstType astType;
 
@@ -227,6 +231,22 @@ public:
         return false;
     }
 
+    virtual QString dump() const
+    {
+        QString r = "Ast(astType=";
+        r.append(astType);
+        r.append(", startLine=");
+        r.append(startLine);
+        r.append(", startCol=");
+        r.append(startCol);
+        r.append(", endCol=");
+        r.append(endCol);
+        r.append(", endLine=");
+        r.append(endLine);
+        r.append(")");
+        return r;
+    };
+
     int startCol;
     int startLine;
     int endCol;
@@ -253,6 +273,8 @@ public:
         return value;
     };
     QString value;
+
+    QString dump() const override;
 };
 
 // this replaces ModuleAst
@@ -263,6 +285,8 @@ public:
     typedef QSharedPointer<CodeAst> Ptr;
     QList<Ast*> body;
     Identifier* name; // module name
+    QString dump() const override;
+
 };
 
 /** Statement classes **/
@@ -280,6 +304,7 @@ public:
     QList<Ast*> body;
     ExpressionAst* returns;
     bool async;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ClassDefinitionAst : public StatementAst {
@@ -289,18 +314,21 @@ public:
     QList<ExpressionAst*> baseClasses;
     QList<Ast*> body;
     QList<ExpressionAst*> decorators;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ReturnAst : public StatementAst {
 public:
     ReturnAst(Ast* parent);
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT DeleteAst : public StatementAst {
 public:
     DeleteAst(Ast* parent);
     QList<ExpressionAst*> targets;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT AssignmentAst : public StatementAst {
@@ -308,6 +336,7 @@ public:
     AssignmentAst(Ast* parent);
     QList<ExpressionAst*> targets;
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT AugmentedAssignmentAst : public StatementAst {
@@ -316,6 +345,7 @@ public:
     ExpressionAst* target;
     Ast::OperatorTypes op;
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT AnnotationAssignmentAst : public StatementAst {
@@ -324,6 +354,7 @@ public:
     ExpressionAst* target;
     ExpressionAst* value;
     ExpressionAst* annotation;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ForAst : public StatementAst {
@@ -333,6 +364,8 @@ public:
     ExpressionAst* iterator;
     QList<Ast*> body;
     QList<Ast*> orelse;
+    bool async;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT WhileAst : public StatementAst {
@@ -341,6 +374,7 @@ public:
     ExpressionAst* condition;
     QList<Ast*> body;
     QList<Ast*> orelse;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT IfAst : public StatementAst {
@@ -349,6 +383,7 @@ public:
     ExpressionAst* condition;
     QList<Ast*> body;
     QList<Ast*> orelse;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT WithItemAst : public Ast {
@@ -363,6 +398,8 @@ public:
     WithAst(Ast* parent);
     QList<Ast*> body;
     QList<WithItemAst*> items;
+    bool async;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT RaiseAst : public StatementAst {
@@ -370,6 +407,7 @@ public:
     RaiseAst(Ast* parent);
     ExpressionAst* type;
     // TODO check what the other things in the grammar actually are and add them
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT TryAst : public StatementAst {
@@ -379,6 +417,7 @@ public:
     QList<ExceptionHandlerAst*> handlers;
     QList<Ast*> orelse;
     QList<Ast*> finally;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT AssertionAst : public StatementAst {
@@ -386,12 +425,23 @@ public:
     AssertionAst(Ast* parent);
     ExpressionAst* condition;
     ExpressionAst* message;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT TryStarAst : public StatementAst {
+public:
+    TryStarAst(Ast* parent);
+    QList<Ast*> body;
+    QList<ExceptionHandlerAst*> handlers;
+    QList<Ast*> orelse;
+    QList<Ast*> finally;
 };
 
 class KDEVPYTHONPARSER_EXPORT ImportAst : public StatementAst {
 public:
     ImportAst(Ast* parent);
     QList<AliasAst*> names;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ImportFromAst : public StatementAst {
@@ -400,12 +450,14 @@ public:
     Identifier* module;
     QList<AliasAst*> names;
     int level;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT GlobalAst : public StatementAst {
 public:
     GlobalAst(Ast* parent);
     QList<Identifier*> names;
+    QString dump() const override;
 };
 
 // TODO what's stmt::Expr(expr value) in the grammar and what do we need it for?
@@ -413,21 +465,25 @@ public:
 class KDEVPYTHONPARSER_EXPORT BreakAst : public StatementAst {
 public:
     BreakAst(Ast* parent);
+    QString dump() const override { return "Break()"; }
 };
 
 class KDEVPYTHONPARSER_EXPORT ContinueAst : public StatementAst {
 public:
     ContinueAst(Ast* parent);
+    QString dump() const override { return "Continue()"; }
 };
 
 class KDEVPYTHONPARSER_EXPORT PassAst : public StatementAst {
 public:
     PassAst(Ast* parent);
+    QString dump() const override { return "Pass()"; }
 };
 
 class KDEVPYTHONPARSER_EXPORT NonlocalAst : public StatementAst {
 public:
     NonlocalAst(Ast* parent);
+    QString dump() const override { return "Nonlocal()"; }
 };
 
 
@@ -444,23 +500,32 @@ public:
     ExpressionAst* value; // WARNING this is not set in most cases!
 };
 
+class KDEVPYTHONPARSER_EXPORT ConstantAst : public ExpressionAst {
+public:
+    ConstantAst(Ast* parent, AstType type = Ast::ConstantAstType): ExpressionAst(parent, type) {}
+    // TODO: Python 3.8 + removed classes ast.Num, ast.Str, ast.Bytes, ast.NameConstant and ast.Ellipsis
+};
+
 class KDEVPYTHONPARSER_EXPORT AssignmentExpressionAst : public ExpressionAst {
 public:
     AssignmentExpressionAst(Ast* parent);
     ExpressionAst* target;
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT AwaitAst : public ExpressionAst {
 public:
     AwaitAst(Ast* parent);
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT YieldFromAst : public ExpressionAst {
 public:
     YieldFromAst(Ast* parent);
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT BooleanOperationAst : public ExpressionAst {
@@ -513,6 +578,8 @@ public:
     UnaryOperationAst(Ast* parent);
     Ast::UnaryOperatorTypes type;
     ExpressionAst* operand;
+
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT LambdaAst : public ExpressionAst {
@@ -528,6 +595,7 @@ public:
     ExpressionAst* condition;
     ExpressionAst* body;
     ExpressionAst* orelse;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT DictAst : public ExpressionAst {
@@ -535,12 +603,14 @@ public:
     DictAst(Ast* parent);
     QList<ExpressionAst*> keys; // WARNING: Can contain null elements: `{**other}`
     QList<ExpressionAst*> values;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT SetAst : public ExpressionAst {
 public:
     SetAst(Ast* parent);
     QList<ExpressionAst*> elements;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ListComprehensionAst : public ExpressionAst {
@@ -587,18 +657,20 @@ public:
     ExpressionAst* value;
 };
 
-class KDEVPYTHONPARSER_EXPORT NumberAst : public ExpressionAst {
+class KDEVPYTHONPARSER_EXPORT NumberAst : public ConstantAst {
 public:
-    NumberAst(Ast* parent);
-    long value; // only used for ints
-    bool isInt; // otherwise it's a float
+    NumberAst(Ast* parent) : ConstantAst(parent, Ast::NumberAstType) {}
+    long value = 0; // only used for ints
+    bool isInt = false; // otherwise it's a float
+    QString dump() const override;
 };
 
-class KDEVPYTHONPARSER_EXPORT StringAst : public ExpressionAst {
+class KDEVPYTHONPARSER_EXPORT StringAst : public ConstantAst {
 public:
-    StringAst(Ast* parent);
-    QString value;
-    bool usedAsComment;
+    StringAst(Ast* parent) : ConstantAst(parent, Ast::StringAstType) {}
+    QString value = "";
+    bool usedAsComment = false;
+    QString dump() const override { return "Str('" + value + "')"; }
 };
 
 class KDEVPYTHONPARSER_EXPORT JoinedStringAst : public ExpressionAst {
@@ -615,16 +687,17 @@ public:
     ExpressionAst* formatSpec;
 };
 
-class KDEVPYTHONPARSER_EXPORT BytesAst : public ExpressionAst {
+class KDEVPYTHONPARSER_EXPORT BytesAst : public ConstantAst {
 public:
-    BytesAst(Ast* parent);
-    QString value;
+    BytesAst(Ast* parent) : ConstantAst(parent, Ast::BytesAstType) {};
+    QString value = "";
 };
 
 class KDEVPYTHONPARSER_EXPORT YieldAst : public ExpressionAst {
 public:
     YieldAst(Ast* parent);
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT NameAst : public ExpressionAst {
@@ -632,18 +705,20 @@ public:
     NameAst(Ast* parent);
     Identifier* identifier;
     ExpressionAst::Context context;
+    QString dump() const override;
 };
 
-class KDEVPYTHONPARSER_EXPORT NameConstantAst : public ExpressionAst {
+class KDEVPYTHONPARSER_EXPORT NameConstantAst : public ConstantAst {
 public:
-    NameConstantAst(Ast* parent);
+    NameConstantAst(Ast* parent) : ConstantAst(parent, Ast::NameConstantAstType) {}
     enum NameConstantTypes {
         False,
         True,
         None,
         Invalid // should not happen
     };
-    NameConstantTypes value;
+    NameConstantTypes value = Invalid;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT CallAst : public ExpressionAst {
@@ -652,6 +727,7 @@ public:
     ExpressionAst* function;
     QList<ExpressionAst*> arguments;
     QList<KeywordAst*> keywords;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT AttributeAst : public ExpressionAst {
@@ -661,6 +737,7 @@ public:
     Identifier* attribute;
     ExpressionAst::Context context;
     int depth;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT SubscriptAst : public ExpressionAst {
@@ -669,6 +746,7 @@ public:
     ExpressionAst* value;
     ExpressionAst* slice;
     ExpressionAst::Context context;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT StarredAst : public ExpressionAst {
@@ -676,6 +754,7 @@ public:
     StarredAst(Ast* parent);
     ExpressionAst* value;
     ExpressionAst::Context context;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ListAst : public ExpressionAst {
@@ -683,6 +762,7 @@ public:
     ListAst(Ast* parent);
     QList<ExpressionAst*> elements;
     ExpressionAst::Context context;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT TupleAst : public ExpressionAst {
@@ -690,11 +770,13 @@ public:
     TupleAst(Ast* parent);
     QList<ExpressionAst*> elements;
     ExpressionAst::Context context;
+    QString dump() const override;
 };
 
-class KDEVPYTHONPARSER_EXPORT EllipsisAst : public ExpressionAst {
+class KDEVPYTHONPARSER_EXPORT EllipsisAst : public ConstantAst {
 public:
-    EllipsisAst(Ast* parent);
+    EllipsisAst(Ast* parent) : ConstantAst(parent, Ast::EllipsisAstType) {};
+    QString dump() const override { return "Ellipsis()"; }
 };
 
 class KDEVPYTHONPARSER_EXPORT SliceAst : public ExpressionAst {
@@ -703,6 +785,7 @@ public:
     ExpressionAst* lower;
     ExpressionAst* upper;
     ExpressionAst* step;
+    QString dump() const override;
 };
 
 
@@ -713,6 +796,7 @@ public:
     ArgAst(Ast* parent);
     Identifier* argumentName;
     ExpressionAst* annotation;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ArgumentsAst : public Ast {
@@ -722,8 +806,10 @@ public:
     QList<ArgAst*> kwonlyargs;
     QList<ArgAst*> posonlyargs;
     QList<ExpressionAst*> defaultValues;
+    QList<ExpressionAst*> defaultKwValues;
     ArgAst* vararg;
     ArgAst* kwarg;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT KeywordAst : public Ast {
@@ -731,6 +817,7 @@ public:
     KeywordAst(Ast* parent);
     Identifier* argumentName;
     ExpressionAst* value;
+    QString dump() const override;
 };
 
 class KDEVPYTHONPARSER_EXPORT ComprehensionAst : public Ast {
@@ -754,8 +841,98 @@ public:
     AliasAst(Ast* parent);
     Identifier* name;
     Identifier* asName;
+    QString dump() const override;
 };
 
-}
+
+/** Match classes **/
+
+class KDEVPYTHONPARSER_EXPORT PatternAst : public Ast {
+public:
+    PatternAst(Ast* parent, AstType type = Ast::PatternAstType): Ast(parent, type) {};
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchCaseAst : public Ast {
+public:
+    MatchCaseAst(Ast* parent) : Ast(parent, Ast::MatchCaseAstType) {};
+    PatternAst* pattern = nullptr;
+    ExpressionAst* guard = nullptr;
+    QList<Ast*> body;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchAst : public Ast {
+public:
+    MatchAst(Ast* parent) : Ast(parent, Ast::MatchAstType) {};
+    ExpressionAst* subject = nullptr;
+    QList<MatchCaseAst*> cases;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchValueAst : public PatternAst {
+public:
+    MatchValueAst(Ast* parent): PatternAst(parent, Ast::MatchValueAstType) {};
+    ExpressionAst* value = nullptr;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchSingletonAst : public PatternAst {
+public:
+    MatchSingletonAst(Ast* parent): PatternAst(parent, Ast::MatchSingletonAstType) {};
+    NameConstantAst::NameConstantTypes value = NameConstantAst::NameConstantTypes::Invalid;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchSequenceAst : public PatternAst {
+public:
+    MatchSequenceAst(Ast* parent) : PatternAst(parent, Ast::MatchSequenceAstType) {};
+    QList<PatternAst*> patterns;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchMappingAst : public PatternAst {
+public:
+    MatchMappingAst(Ast* parent) : PatternAst(parent, Ast::MatchMappingAstType) {};
+    QList<ExpressionAst*> keys;
+    Identifier *rest = nullptr;
+    QList<PatternAst*> patterns;
+    QString dump() const override;
+};
+
+
+class KDEVPYTHONPARSER_EXPORT MatchClassAst : public PatternAst {
+public:
+    MatchClassAst(Ast* parent): PatternAst(parent, Ast::MatchClassAstType) {};
+    ExpressionAst* cls = nullptr;
+    QList<PatternAst*> patterns;
+    Identifier *kwdAttrs = nullptr;
+    QList<PatternAst*> kwdPatterns;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchStarAst : public PatternAst {
+public:
+    MatchStarAst(Ast* parent): PatternAst(parent, Ast::MatchStarAstType) {};
+    Identifier* name = nullptr;
+    QString dump() const override;
+};
+
+class KDEVPYTHONPARSER_EXPORT MatchAsAst : public PatternAst {
+public:
+    MatchAsAst(Ast* parent): PatternAst(parent, Ast::MatchAsAstType) {};
+    PatternAst* pattern = nullptr;
+    Identifier* name = nullptr;
+    QString dump() const override;
+};
+
+
+class KDEVPYTHONPARSER_EXPORT MatchOrAst : public PatternAst {
+public:
+    MatchOrAst(Ast* parent): PatternAst(parent, Ast::MatchOrAstType) {};
+    QList<PatternAst*> patterns;
+    QString dump() const override;
+};
+
+} // end namespace Python
 
 #endif
